# Copyright 2021- Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)
import base64
import logging
from datetime import datetime

from odoo import fields, models
from odoo.tools import html_sanitize

_logger = logging.getLogger(__name__)


class EbicsStatementRopImport(models.TransientModel):
    _name = "ebics.statement.import.wizard"
    _description = "Wizard d'import des fichiers EBICS et ROP"

    date_from = fields.Date()
    date_to = fields.Date()
    log = fields.Html()

    def load_ebics(self):
        self.log = ""
        ebics_mvt = self.env["ebics.file.format"].search(
            [("name", "=", "camt.xxx.cfonb120.stm.0by")]
        )
        ebics_rop = self.env["ebics.file.format"].search(
            [("name", "=", "camt.xxx.bnpp300.0dh")]
        )
        ebics_conf_id = self.env["ebics.xfer"]._default_ebics_config_id()
        avail_userids = ebics_conf_id.ebics_userid_ids.filtered(
            lambda r: self.env.user.id in r.user_ids.ids
        )
        if avail_userids and len(avail_userids) == 1:
            user_id = avail_userids[0]
        else:
            user_id = False

        # Download EBICS MVT
        _logger.debug("Creating wizard for retrieving enriched moves")
        ebics_mvt_obj = self.env["ebics.xfer"].create(
            {
                "ebics_config_id": ebics_conf_id.id,
                "ebics_userid_id": user_id.id,
                "format_id": ebics_mvt.id,
                "date_from": self.date_from,
                "date_to": self.date_to,
            }
        )
        ebics_wizard_act = ebics_mvt_obj.ebics_download()
        self.log += html_sanitize(
            "<strong><u>Téléchargement des mouvements enrichis</u></strong><br/>"
            f"{ebics_mvt_obj.note}"
            "<hr/>"
        )
        ebics_mvt_file_id = ebics_wizard_act.get("context").get("ebics_file_ids")
        ebics_mvt_file = self.env["ebics.file"].browse(ebics_mvt_file_id)
        if ebics_mvt_file:
            # Process EBICS MVT (= create bank statement)
            _logger.debug("Creating statements for retrieved enriched moves")
            ebics_mvt_file.with_context(active_model="ebics.file").process()

            bank_statement = ebics_mvt_file.bank_statement_ids
            bank_statement.ensure_one()
            _logger.debug(f"Created statement : {bank_statement.date}")
            self.log += html_sanitize(
                "<strong><u>Import des mouvements enrichis</u></strong><br/>"
                f"Référence : {bank_statement.name}<br/>"
                f" - Journal : {bank_statement.journal_id.name}<br />"
                f" - Date : {bank_statement.date} <br/>"
                f"Solde Initial : {bank_statement.balance_start} €<br/>"
                f"Solde Final : {bank_statement.balance_end} €"
                "<hr/>"
            )
        else:
            _logger.debug("No enriched moves file, no creating statements")
            self.log += html_sanitize(
                "<strong><u>Import des mouvements enrichis</u></strong><br/>"
                "Aucune donnée n'a pu être téléchargée, "
                "il n'y a donc pas de données à importer.<hr/>"
            )

        # Download EBICS ROP
        _logger.debug("Creating wizard for retrieving ROP")
        ebics_rop_obj = self.env["ebics.xfer"].create(
            {
                "ebics_config_id": ebics_conf_id.id,
                "ebics_userid_id": user_id.id,
                "format_id": ebics_rop.id,
                "date_from": self.date_from,
                "date_to": self.date_to,
            }
        )
        ebics_rop_wizard_act = ebics_rop_obj.ebics_download()
        self.log += html_sanitize(
            "<strong><u>Téléchargement des mouvements ROP</u></strong><br/>"
            f"{ebics_rop_obj.note}"
            "<hr/>"
        )
        ebics_rop_file_id = ebics_rop_wizard_act.get("context").get("ebics_file_ids")
        ebics_rop_file = self.env["ebics.file"].browse(ebics_rop_file_id)

        if ebics_rop_file and bank_statement:
            _logger.debug(f"Processing ROP file {ebics_rop_file.name}")
            # Process EBICS ROP file
            ebics_rop_file.ensure_one()
            # Suppression de lignes CORPORATE
            lines_to_delete = bank_statement.line_ids.filtered(
                lambda line: "CORPORATE CARD" in line.payment_ref
            )
            line_nb = len(lines_to_delete)
            _logger.debug(f"Deleting {line_nb} statement lines : {lines_to_delete.ids}")
            lines_to_delete.unlink()
            self.log += html_sanitize(
                "<strong><u>Suppression de lignes CORPORATE</u></strong><br/>"
                f"Suppression de {line_nb} lignes."
                "<hr/>"
            )

            # Ajout des lignes ROP
            _logger.debug("Adding statement lines for ROP files")
            rop_lines = self._get_rop_lines(ebics_rop_file.data)
            for ropl in rop_lines:
                _logger.debug(f"Adding {ropl['payment_ref']} - {ropl['date']}")
                ropl.update(
                    {
                        "statement_id": bank_statement.id,
                        "journal_id": ebics_conf_id.journal_ids[0].id,
                    }
                )
                self.env["account.bank.statement.line"].create(ropl)
            ebics_rop_file.set_to_done()

            self.log += html_sanitize(
                "<strong><u>Création des lignes ROP</u></strong><br/>"
                f"Création de {len(rop_lines)} lignes."
                "<hr/>"
            )
        else:
            _logger.debug(
                "No ROP file to process or no bank_statements created by enriched moves"
            )
            self.log += html_sanitize(
                "<strong><u>Suppression de lignes CORPORATE & "
                "Intégration du fichier ROP</u></strong><br/>"
                "Il n'y a pas de fichier ROP ou Mouvement disponible "
                "donc les lignes <i>CORPORATE</i>"
                "ne peuvent être supprimées et le fichier ROP ne peut être intégré."
                "<hr/>"
            )

        result_view = self.env.ref(
            "3adentaire_account_ebics.ebics_statement_import_wizard_form_result"
        )
        return {
            "name": "Résultat du chargement automatique",
            "res_id": self.id,
            "view_mode": "form",
            "res_model": "ebics.statement.import.wizard",
            "view_id": result_view.id,
            "context": {"statement_id": bank_statement.id if ebics_mvt_file else None},
            "target": "new",
            "type": "ir.actions.act_window",
        }

    def view_statement(self):
        statement_id = self.env.context.get("statement_id")
        return {
            "res_id": statement_id,
            "view_mode": "form",
            "res_model": "account.bank.statement",
            "type": "ir.actions.act_window",
        }

    def _get_rop_lines(self, data):
        """
        Parse ROP file to return lines
        """
        rop_lines = []
        content = base64.b64decode(data).decode("utf-8").split("\n")
        for line in content:
            if line[0:2] in ("11", "21"):
                multiplier = 1 if line[87] == "C" else -1
                amount = int(line[75:87]) * multiplier / 100
            elif line[0:2] in ("12", "22"):
                multiplier = 1 if line[86] == "C" else -1
                amount = int(line[75:86]) * multiplier / 100
            else:
                continue

            rop_lines.append(
                {
                    "date": datetime.strptime(line[177:185], "%Y%m%d").date(),
                    "payment_ref": line[29:61].rstrip()
                    + " - "
                    + line[138:158].rstrip()
                    + " - "
                    + line[104:128].rstrip(),
                    "amount": amount,
                }
            )
        return rop_lines
